
import sys
sys.path.append('../')
import numpy as np


from crazy_mesh import CrazyMesh
from mimetic_basis_polynomials import MimeticBasisPolynomials
from projection import Reconstruction

import matplotlib.pyplot as plt
from matplotlib import cm


filenames = ["hdMSEM_K4_N4_c0_du_plus_f.txt",
             "hdMSEM_K4_N4_c125_du_plus_f.txt",
             "hdMSEM_K4_N4_c250_du_plus_f.txt",
             ]
M = [0,3] # we will plot the middle surfaces of element on the mth level along  z-axis


# filenames = ["hMSEM_K4_N4_c0_du_plus_f.txt",
#              "hMSEM_K4_N4_c125_du_plus_f.txt",
#              "hMSEM_K4_N4_c250_du_plus_f.txt",
#              ]
# M = [0,3] # we will plot the middle surfaces of element on the mth level along  z-axis

# filenames = ["MSEM_K2_N4_c0_du_plus_f.txt",
#              "MSEM_K2_N4_c125_du_plus_f.txt",
#              "MSEM_K2_N4_c250_du_plus_f.txt",
#              ]
# M = [0,1] # we will plot the middle surfaces of element on the mth level along  z-axis


density = 30 # how refined the plot will be
do_save = True







for filename in filenames:
    saveto = filename.split('.')[0] + '.pdf'
    duf = np.loadtxt(filename)
    parse_filename = filename.split('_')
    K = parse_filename[1]
    N = parse_filename[2]
    c = parse_filename[3]

    K = int(K[1:])
    N = int(N[1:])
    c = int(c[1:])/1000
    assert duf.shape == (K**3, N**3)

    mesh = CrazyMesh(c, K)
    _bfN_ = 'Lobatto-' + str(N)
    mbf = MimeticBasisPolynomials(_bfN_, _bfN_, _bfN_)

    elements_global_numbering = np.arange(K**3, dtype='int').reshape((K, K, K), order='F')

    XL = list()
    YL = list()
    ZL = list()
    VL = list()
    for m in M:
        elements = elements_global_numbering[:, :, m].ravel('F')

        dofs = list()
        for e in elements:
            local_dofs = duf[e]
            dofs.append(local_dofs)


        _ = np.linspace(-1,1,density)
        xi = (_ + 1/(density-1))[:-1]
        et = (_ + 1/(density-1))[:-1]
        sg = np.array([0,])

        LEN = len(xi)
        X = np.zeros((LEN*K, LEN*K))
        Y = np.zeros((LEN*K, LEN*K))
        Z = np.zeros((LEN*K, LEN*K))
        V = np.zeros((LEN*K, LEN*K))
        for j in range(K):
            for i in range(K):
                _ = i + j * K
                dof = dofs[_]
                ct = mesh.CT_of_element_index(i, j,m)
                Rc = Reconstruction(mbf, ct)
                xyz, v = Rc.VP(dof, xi, et, sg)
                x, y, z = xyz
                X[i*LEN:(i+1)*LEN, j*LEN:(j+1)*LEN] = x[:,:,0]
                Y[i*LEN:(i+1)*LEN, j*LEN:(j+1)*LEN] = y[:,:,0]
                Z[i*LEN:(i+1)*LEN, j*LEN:(j+1)*LEN] = z[:,:,0]
                V[i*LEN:(i+1)*LEN, j*LEN:(j+1)*LEN] = v[:,:,0]

        V = np.log10(np.abs(V))

        VL.append(V)
        XL.append(X)
        YL.append(Y)
        ZL.append(Z)

    if filename[:2] == 'hd':
        upper_bound = -10
        lower_bound = -15
    else:
        upper_bound = -11
        lower_bound = -16

    VL = np.array(VL)
    VL[VL < lower_bound] = lower_bound
    MAX = np.max(VL)
    MIN = np.min(VL)

    # 3d surface plot...
    scale = 1/(MAX - MIN)
    VL -= MIN
    VL *= scale


    ratio = (MAX-MIN) / (upper_bound-MIN)
    VL *= ratio

    tick_range = upper_bound - lower_bound
    ticks = [_ for _ in range(lower_bound, upper_bound+1, 1)]

    plt.rc('text', usetex=True)
    plt.rcParams['text.latex.preamble'] = r"\usepackage{amsmath}"

    fig = plt.figure(figsize=(8, 5.5))
    ax = fig.add_subplot(111, projection='3d')
    # make the panes transparent
    ax.xaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))
    ax.yaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))
    ax.zaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))
    # make the grid lines transparent
    ax.xaxis._axinfo["grid"]['color'] =  (1,1,1,0)
    ax.yaxis._axinfo["grid"]['color'] =  (1,1,1,0)
    ax.zaxis._axinfo["grid"]['color'] =  (1,1,1,0)
    ax.view_init(12,45)
    cmap = cm.coolwarm

    for i,_ in enumerate(M):
        ax.plot_surface(XL[i], YL[i], ZL[i], facecolors=cmap(VL[i]))


    mappable = cm.ScalarMappable(cmap=cmap)
    mappable.set_array(np.array(ticks))
    cb = plt.colorbar(mappable, ax=ax, #ticks=np.linspace(0,1,num_ticks),
                        shrink=0.8, aspect=17,
                        extend='min',
                        orientation='vertical',
                        )
    cb.set_label(r'$ \log_{10}\left( \left| \nabla\cdot\boldsymbol{u}^h+f^h \right| \right) $', labelpad=10, size=15)
    cb.ax.tick_params(labelsize=13.5)
    # cb.ax.set_yticklabels(
    #     [f"$%i$"%ticks[i] for i in range(num_ticks)]
    #     )  # vertically oriented colorbar

    ax.plot([0,1],[0,0],[0,0], color=(0,0,0,0.3), linewidth=1)
    ax.plot([0,1],[1,1],[0,0], color=(0,0,0,0.3), linewidth=1)
    ax.plot([0,1],[0,0],[1,1], color=(0,0,0,0.3), linewidth=1)
    ax.plot([0,1],[1,1],[1,1], color=(0,0,0,0.3), linewidth=1)
    ax.plot([0,0],[0,1],[0,0], color=(0,0,0,0.3), linewidth=1)
    ax.plot([1,1],[0,1],[0,0], color=(0,0,0,0.3), linewidth=1)
    ax.plot([0,0],[0,1],[1,1], color=(0,0,0,0.3), linewidth=1)
    ax.plot([1,1],[0,1],[1,1], color=(0,0,0,0.3), linewidth=1)
    ax.plot([0,0],[0,0],[0,1], color=(0,0,0,0.3), linewidth=1)
    ax.plot([0,0],[1,1],[0,1], color=(0,0,0,0.3), linewidth=1)
    ax.plot([1,1],[0,0],[0,1], color=(0,0,0,0.3), linewidth=1)
    ax.plot([1,1],[1,1],[0,1], color=(0,0,0,0.3), linewidth=1)

    ax.tick_params(labelsize=12)
    ax.set_xlabel(r'$x$', fontsize=15)
    ax.set_ylabel(r'$y$', fontsize=15)
    ax.set_zlabel(r'$z$', fontsize=15)

    # plt.title(f"$c={c}$", fontsize=15)
    if do_save:
        plt.savefig(saveto, bbox_inches='tight')
    plt.show()
    plt.close()
