"""
Here we present results of the Poisson problem using the MSEM.

⭕ To access the source code, click on the [source] button at the right
side or click on
:download:`[results.py]</_contents/LIBRARY/ptc/mathischeap_ptc/results.py>`.
Dependence may exist. In case of error, check import and install required
packages or download required scripts. © mathischeap.com

"""

import numpy as np

h_filename = "hdMSEM_h_convergence.txt"
p_filename = "hdMSEM_p_convergence.txt"


if h_filename[:2] == 'MS':
    h_what = 'MSEM'
elif h_filename[:2] == 'hM':
    h_what = 'hMSEM'
elif h_filename[:2] == 'hd':
    h_what = 'hdMSEM'
else:
    raise Exception()

if p_filename[:2] == 'MS':
    p_what = 'MSEM'
elif p_filename[:2] == 'hM':
    p_what = 'hMSEM'
elif p_filename[:2] == 'hd':
    p_what = 'hdMSEM'
else:
    raise Exception()

h_convergence_results = np.loadtxt(h_filename)
p_convergence_results = np.loadtxt(p_filename)




group_h = True
group_p = True

if group_h: # group h convergence data ............
    N1_c0 = list()
    for hd in h_convergence_results:
        if hd[1] == 1 and hd[2] == 0:
            N1_c0.append(hd)
    N1_c0 = np.array(N1_c0)

    N1_c125 = list()
    for hd in h_convergence_results:
        if hd[1] == 1 and hd[2] == 0.125:
            N1_c125.append(hd)
    N1_c125 = np.array(N1_c125)

    N1_c25 = list()
    for hd in h_convergence_results:
        if hd[1] == 1 and hd[2] == 0.25:
            N1_c25.append(hd)
    N1_c25 = np.array(N1_c25)



    N2_c0 = list()
    for hd in h_convergence_results:
        if hd[1] == 2 and hd[2] == 0:
            N2_c0.append(hd)
    N2_c0 = np.array(N2_c0)

    N2_c125 = list()
    for hd in h_convergence_results:
        if hd[1] == 2 and hd[2] == 0.125:
            N2_c125.append(hd)
    N2_c125 = np.array(N2_c125)

    N2_c25 = list()
    for hd in h_convergence_results:
        if hd[1] == 2 and hd[2] == 0.25:
            N2_c25.append(hd)
    N2_c25 = np.array(N2_c25)



    N3_c0 = list()
    for hd in h_convergence_results:
        if hd[1] == 3 and hd[2] == 0:
            N3_c0.append(hd)
    N3_c0 = np.array(N3_c0)

    N3_c125 = list()
    for hd in h_convergence_results:
        if hd[1] == 3 and hd[2] == 0.125:
            N3_c125.append(hd)
    N3_c125 = np.array(N3_c125)

    N3_c25 = list()
    for hd in h_convergence_results:
        if hd[1] == 3 and hd[2] == 0.25:
            N3_c25.append(hd)
    N3_c25 = np.array(N3_c25)
else:
    raise Exception()

if group_p: # group p convergence data .....
    K1_c0 = list()
    for pd in p_convergence_results:
        if pd[0] == 1 and pd[2] == 0:
            K1_c0.append(pd)
    K1_c0 = np.array(K1_c0)

    K1_c125 = list()
    for pd in p_convergence_results:
        if pd[0] == 1 and pd[2] == 0.125:
            K1_c125.append(pd)
    K1_c125 = np.array(K1_c125)
    K1_c25 = list()
    for pd in p_convergence_results:
        if pd[0] == 1 and pd[2] == 0.25:
            K1_c25.append(pd)
    K1_c25 = np.array(K1_c25)


    K2_c0 = list()
    for pd in p_convergence_results:
        if pd[0] == 2 and pd[2] == 0:
            K2_c0.append(pd)
    K2_c0 = np.array(K2_c0)
    K2_c125 = list()
    for pd in p_convergence_results:
        if pd[0] == 2 and pd[2] == 0.125:
            K2_c125.append(pd)
    K2_c125 = np.array(K2_c125)
    K2_c25 = list()
    for pd in p_convergence_results:
        if pd[0] == 2 and pd[2] == 0.25:
            K2_c25.append(pd)
    K2_c25 = np.array(K2_c25)


    K3_c0 = list()
    for pd in p_convergence_results:
        if pd[0] == 3 and pd[2] == 0:
            K3_c0.append(pd)
    K3_c0 = np.array(K3_c0)
    K3_c125 = list()
    for pd in p_convergence_results:
        if pd[0] == 3 and pd[2] == 0.125:
            K3_c125.append(pd)
    K3_c125 = np.array(K3_c125)
    K3_c25 = list()
    for pd in p_convergence_results:
        if pd[0] == 3 and pd[2] == 0.25:
            K3_c25.append(pd)
    K3_c25 = np.array(K3_c25)
else:
    raise Exception()

def ___plot_MTIR_plot_order_triangle___(plot_type, x_range,
                                        potn, xd, yd):
    # ___default "p"________________________________________________________________
    if "p" not in potn:
        potn["p"] = (0, -0.3)
    if "tp" not in potn:
        potn["tp"] = (0.02, 0.2)
    # ___default "l"________________________________________________________________
    if "l" not in potn:
        potn["l"] = 0.1
    # ___default "h"________________________________________________________________
    if "order" not in potn:
        potn["order"] = (np.log10(yd[-1]) - np.log10(yd[-2])) / (
                    np.log10(xd[-1]) - np.log10(xd[-2]))
    # ___ loglog____________________________________________________________________
    if plot_type == 'loglog':
        x_max, xmin = x_range
        x_range = np.log10(x_max) - np.log10(xmin)
        origin = (xd[-1], yd[-1])
        otc0x = np.log10(origin[0]) + x_range * potn["p"][0]
        otc0x = 10 ** otc0x
        otc0y = np.log10(origin[1]) + x_range * potn["p"][1]
        otc0y = 10 ** otc0y
        otc0 = (otc0x, otc0y)  # order_triangle_corner_0
        otc1x = np.log10(otc0x) + x_range * potn["l"]
        otc1x = 10 ** otc1x
        otc1 = (otc1x, otc0y)  # order_triangle_corner_0
        otc2y = np.log10(otc0y) + x_range * potn["l"] * potn[
            "order"]
        otc2y = 10 ** otc2y
        otc2 = (otc1x, otc2y)  # order_triangle_corner_0
        ttps_x, ttps_y = potn["tp"]
        textpos_x = 10 ** (np.log10(otc1x) + x_range * ttps_x)
        textpos_y = 10 ** (
                    np.log10(otc0y) + x_range * potn["l"] * potn[
                "order"] * ttps_y)
        order = potn["order"]
        return otc0, otc1, otc2, (textpos_x, textpos_y), order
    # ___ELSE: ERRORING_____________________________________________________________
    else:
        raise NotImplementedError(
            " <plot_order_triangle> does not work for {} plot".format(
                plot_type))


do_save = True

usetex = True
import matplotlib.pyplot as plt

plt.rc('text', usetex=usetex)
if usetex: plt.rcParams['text.latex.preamble'] = r"\usepackage{amsmath}"


# u L_2 error :  h-convergence
K_index = 0
v_index = 3
plot_hc_u_L_2_error = 0

if plot_hc_u_L_2_error:
    plt.figure(figsize=(6,4.5))
    plt.gcf().subplots_adjust(left=0.15)
    plt.gcf().subplots_adjust(bottom=0.15)

    h_data = 1/N1_c0[:,K_index]
    v_data = N1_c0[:,v_index]
    # order = (np.log10(v_data[-1])- np.log10(v_data[-2])) /\
    #                     (np.log10(h_data[-1])-np.log10(h_data[-2]))
    plt.loglog(h_data, v_data, '-o', color=(0,0,0,1), linewidth=0.8)


    xd_max = np.max(h_data)
    xd_min = np.min(h_data)
    x_range = (xd_max, xd_min)
    c0, c1, c2, textpos, ordern = ___plot_MTIR_plot_order_triangle___(
            'loglog', x_range, {'tp':(0.02, 0.05),'order':1, "p": (0, -0.2)}, h_data, v_data)
    #___plot triangle______________________________________________
    c0x, c0y = c0
    c1x, c1y = c1
    c2x, c2y = c2
    plt.fill_between([c0x, c1x], [c0y, c1y], [c0y, c2y], color='grey', alpha=0.5)
    if isinstance(ordern, int):
        plt.text(textpos[0], textpos[1], "${}$".format(ordern), fontsize=15)
    else:
        plt.text(textpos[0], textpos[1], "${}$".format('%0.2f'%ordern), fontsize=15)


    h_data = 1/N1_c125[:,K_index]
    v_data = N1_c125[:,v_index]
    plt.loglog(h_data, v_data, '-v', color=(0,0,0,0.6), linewidth=0.8)


    h_data = 1/N1_c25[:,K_index]
    v_data = N1_c25[:,v_index]
    plt.loglog(h_data, v_data, '-s', color=(0,0,0,0.3), linewidth=0.8)



    h_data = 1/N2_c0[:,K_index]
    v_data = N2_c0[:,v_index]
    plt.loglog(h_data, v_data, '-o', color=(0,0,1,1), linewidth=0.8)

    x_range = (xd_max, xd_min)
    c0, c1, c2, textpos, ordern = ___plot_MTIR_plot_order_triangle___(
            'loglog', x_range, {'tp':(0.02, 0.2),'order':2, "p": (0, -0.2)}, h_data, v_data)
    #___plot triangle______________________________________________
    c0x, c0y = c0
    c1x, c1y = c1
    c2x, c2y = c2
    plt.fill_between([c0x, c1x], [c0y, c1y], [c0y, c2y], color='grey', alpha=0.5)
    if isinstance(ordern, int):
        plt.text(textpos[0], textpos[1], "${}$".format(ordern), fontsize=15)
    else:
        plt.text(textpos[0], textpos[1], "${}$".format('%0.2f'%ordern), fontsize=15)


    h_data = 1/N2_c125[:,K_index]
    v_data = N2_c125[:,v_index]
    plt.loglog(h_data, v_data, '-v', color=(0,0,1,0.6), linewidth=0.8)



    h_data = 1/N2_c25[:,K_index]
    v_data = N2_c25[:,v_index]
    plt.loglog(h_data, v_data, '-s', color=(0,0,1,0.3), linewidth=0.8)




    h_data = 1/N3_c0[:,K_index]
    v_data = N3_c0[:,v_index]
    plt.loglog(h_data, v_data, '-o', color=(1,0,0,1), linewidth=0.8)

    x_range = (xd_max, xd_min)
    c0, c1, c2, textpos, ordern = ___plot_MTIR_plot_order_triangle___(
            'loglog', x_range, {'tp':(0.02, 0.2),'order':3, "p": (0, -0.3)}, h_data, v_data)
    #___plot triangle______________________________________________
    c0x, c0y = c0
    c1x, c1y = c1
    c2x, c2y = c2
    plt.fill_between([c0x, c1x], [c0y, c1y], [c0y, c2y], color='grey', alpha=0.5)
    if isinstance(ordern, int):
        plt.text(textpos[0], textpos[1], "${}$".format(ordern), fontsize=15)
    else:
        plt.text(textpos[0], textpos[1], "${}$".format('%0.2f'%ordern), fontsize=15)



    h_data = 1/N3_c125[:,K_index]
    v_data = N3_c125[:,v_index]
    plt.loglog(h_data, v_data, '-v', color=(1,0,0,0.6), linewidth=0.8)



    h_data = 1/N3_c25[:,K_index]
    v_data = N3_c25[:,v_index]
    plt.loglog(h_data, v_data, '-s', color=(1,0,0,0.3), linewidth=0.8)





    plt.tick_params(which='both', labeltop=False, labelright=False, top=True, right=True)
    plt.tick_params(axis='both', which='minor', direction='in', length=6)
    plt.tick_params(axis='both', which='major', direction='in', length=12)
    plt.tick_params(axis='both', which='both', labelsize=20)
    plt.tick_params(axis='x', which='both', pad=5)
    plt.xlabel('$h$', fontsize=20)
    plt.ylabel(r'$\left\| \boldsymbol{u}^h \right\|_{L^2\text{-}\mathrm{error}}$', fontsize=20)
    # plt.legend(fontsize=13, loc='best', frameon=False)

    plt.yticks([1e0, 1e-1, 1e-2, 1e-3])
    if do_save:
        plt.savefig(h_what+'_h_convergence_u_L_2.pdf', bbox_inches='tight')
    plt.show()
    plt.close()


# u H_div error :  h-convergence
K_index = 0
v_index = 4
plot_hc_u_H_div_error = 0

if plot_hc_u_H_div_error:
    plt.figure(figsize=(6,4.5))
    plt.gcf().subplots_adjust(left=0.15)
    plt.gcf().subplots_adjust(bottom=0.15)

    h_data = 1/N1_c0[:,K_index]
    v_data = N1_c0[:,v_index]
    # order = (np.log10(v_data[-1])- np.log10(v_data[-2])) /\
    #                     (np.log10(h_data[-1])-np.log10(h_data[-2]))
    plt.loglog(h_data, v_data, '-o', color=(0,0,0,1), linewidth=0.8)


    xd_max = np.max(h_data)
    xd_min = np.min(h_data)
    x_range = (xd_max, xd_min)
    c0, c1, c2, textpos, ordern = ___plot_MTIR_plot_order_triangle___(
            'loglog', x_range, {'tp':(0.02, 0.05),'order':1, "p": (0, -0.2)}, h_data, v_data)
    #___plot triangle______________________________________________
    c0x, c0y = c0
    c1x, c1y = c1
    c2x, c2y = c2
    plt.fill_between([c0x, c1x], [c0y, c1y], [c0y, c2y], color='grey', alpha=0.5)
    if isinstance(ordern, int):
        plt.text(textpos[0], textpos[1], "${}$".format(ordern), fontsize=15)
    else:
        plt.text(textpos[0], textpos[1], "${}$".format('%0.2f'%ordern), fontsize=15)


    h_data = 1/N1_c125[:,K_index]
    v_data = N1_c125[:,v_index]
    plt.loglog(h_data, v_data, '-v', color=(0,0,0,0.6), linewidth=0.8)


    h_data = 1/N1_c25[:,K_index]
    v_data = N1_c25[:,v_index]
    plt.loglog(h_data, v_data, '-s', color=(0,0,0,0.3), linewidth=0.8)



    h_data = 1/N2_c0[:,K_index]
    v_data = N2_c0[:,v_index]
    plt.loglog(h_data, v_data, '-o', color=(0,0,1,1), linewidth=0.8)

    x_range = (xd_max, xd_min)
    c0, c1, c2, textpos, ordern = ___plot_MTIR_plot_order_triangle___(
            'loglog', x_range, {'tp':(0.02, 0.2),'order':2, "p": (0, -0.2)}, h_data, v_data)
    #___plot triangle______________________________________________
    c0x, c0y = c0
    c1x, c1y = c1
    c2x, c2y = c2
    plt.fill_between([c0x, c1x], [c0y, c1y], [c0y, c2y], color='grey', alpha=0.5)
    if isinstance(ordern, int):
        plt.text(textpos[0], textpos[1], "${}$".format(ordern), fontsize=15)
    else:
        plt.text(textpos[0], textpos[1], "${}$".format('%0.2f'%ordern), fontsize=15)


    h_data = 1/N2_c125[:,K_index]
    v_data = N2_c125[:,v_index]
    plt.loglog(h_data, v_data, '-v', color=(0,0,1,0.6), linewidth=0.8)



    h_data = 1/N2_c25[:,K_index]
    v_data = N2_c25[:,v_index]
    plt.loglog(h_data, v_data, '-s', color=(0,0,1,0.3), linewidth=0.8)




    h_data = 1/N3_c0[:,K_index]
    v_data = N3_c0[:,v_index]
    plt.loglog(h_data, v_data, '-o', color=(1,0,0,1), linewidth=0.8)

    x_range = (xd_max, xd_min)
    c0, c1, c2, textpos, ordern = ___plot_MTIR_plot_order_triangle___(
            'loglog', x_range, {'tp':(0.02, 0.2),'order':3, "p": (0, -0.3)}, h_data, v_data)
    #___plot triangle______________________________________________
    c0x, c0y = c0
    c1x, c1y = c1
    c2x, c2y = c2
    plt.fill_between([c0x, c1x], [c0y, c1y], [c0y, c2y], color='grey', alpha=0.5)
    if isinstance(ordern, int):
        plt.text(textpos[0], textpos[1], "${}$".format(ordern), fontsize=15)
    else:
        plt.text(textpos[0], textpos[1], "${}$".format('%0.2f'%ordern), fontsize=15)



    h_data = 1/N3_c125[:,K_index]
    v_data = N3_c125[:,v_index]
    plt.loglog(h_data, v_data, '-v', color=(1,0,0,0.6), linewidth=0.8)



    h_data = 1/N3_c25[:,K_index]
    v_data = N3_c25[:,v_index]
    plt.loglog(h_data, v_data, '-s', color=(1,0,0,0.3), linewidth=0.8)





    plt.tick_params(which='both', labeltop=False, labelright=False, top=True, right=True)
    plt.tick_params(axis='both', which='minor', direction='in', length=6)
    plt.tick_params(axis='both', which='major', direction='in', length=12)
    plt.tick_params(axis='both', which='both', labelsize=20)
    plt.tick_params(axis='x', which='both', pad=5)
    plt.xlabel('$h$', fontsize=20)
    plt.ylabel(r'$\left\| \boldsymbol{u}^h \right\|_{H(\mathrm{div})\text{-}\mathrm{error}}$', fontsize=20)
    # plt.legend(fontsize=13, loc='best', frameon=False)

    plt.yticks([1e1, 1e0, 1e-1, 1e-2])
    if do_save:
        plt.savefig(h_what+'_h_convergence_u_H_div.pdf', bbox_inches='tight')
    plt.show()
    plt.close()




# phi L_2 error :  h-convergence
K_index = 0
v_index = 5
plot_hc_phi_L_2_error = 0

if plot_hc_phi_L_2_error:
    plt.figure(figsize=(6,4.5))
    plt.gcf().subplots_adjust(left=0.15)
    plt.gcf().subplots_adjust(bottom=0.15)

    h_data = 1/N1_c0[:,K_index]
    v_data = N1_c0[:,v_index]
    # order = (np.log10(v_data[-1])- np.log10(v_data[-2])) /\
    #                     (np.log10(h_data[-1])-np.log10(h_data[-2]))
    plt.loglog(h_data, v_data, '-o', color=(0,0,0,1), linewidth=0.8)


    xd_max = np.max(h_data)
    xd_min = np.min(h_data)
    x_range = (xd_max, xd_min)
    c0, c1, c2, textpos, ordern = ___plot_MTIR_plot_order_triangle___(
            'loglog', x_range, {'tp':(0.02, 0.05),'order':1, "p": (0, -0.2)}, h_data, v_data)
    #___plot triangle______________________________________________
    c0x, c0y = c0
    c1x, c1y = c1
    c2x, c2y = c2
    plt.fill_between([c0x, c1x], [c0y, c1y], [c0y, c2y], color='grey', alpha=0.5)
    if isinstance(ordern, int):
        plt.text(textpos[0], textpos[1], "${}$".format(ordern), fontsize=15)
    else:
        plt.text(textpos[0], textpos[1], "${}$".format('%0.2f'%ordern), fontsize=15)


    h_data = 1/N1_c125[:,K_index]
    v_data = N1_c125[:,v_index]
    plt.loglog(h_data, v_data, '-v', color=(0,0,0,0.6), linewidth=0.8)


    h_data = 1/N1_c25[:,K_index]
    v_data = N1_c25[:,v_index]
    plt.loglog(h_data, v_data, '-s', color=(0,0,0,0.3), linewidth=0.8)



    h_data = 1/N2_c0[:,K_index]
    v_data = N2_c0[:,v_index]
    plt.loglog(h_data, v_data, '-o', color=(0,0,1,1), linewidth=0.8)

    x_range = (xd_max, xd_min)
    c0, c1, c2, textpos, ordern = ___plot_MTIR_plot_order_triangle___(
            'loglog', x_range, {'tp':(0.02, 0.2),'order':2, "p": (0, -0.2)}, h_data, v_data)
    #___plot triangle______________________________________________
    c0x, c0y = c0
    c1x, c1y = c1
    c2x, c2y = c2
    plt.fill_between([c0x, c1x], [c0y, c1y], [c0y, c2y], color='grey', alpha=0.5)
    if isinstance(ordern, int):
        plt.text(textpos[0], textpos[1], "${}$".format(ordern), fontsize=15)
    else:
        plt.text(textpos[0], textpos[1], "${}$".format('%0.2f'%ordern), fontsize=15)


    h_data = 1/N2_c125[:,K_index]
    v_data = N2_c125[:,v_index]
    plt.loglog(h_data, v_data, '-v', color=(0,0,1,0.6), linewidth=0.8)



    h_data = 1/N2_c25[:,K_index]
    v_data = N2_c25[:,v_index]
    plt.loglog(h_data, v_data, '-s', color=(0,0,1,0.3), linewidth=0.8)




    h_data = 1/N3_c0[:,K_index]
    v_data = N3_c0[:,v_index]
    plt.loglog(h_data, v_data, '-o', color=(1,0,0,1), linewidth=0.8)

    x_range = (xd_max, xd_min)
    c0, c1, c2, textpos, ordern = ___plot_MTIR_plot_order_triangle___(
            'loglog', x_range, {'tp':(0.02, 0.2),'order':3, "p": (0, -0.3)}, h_data, v_data)
    #___plot triangle______________________________________________
    c0x, c0y = c0
    c1x, c1y = c1
    c2x, c2y = c2
    plt.fill_between([c0x, c1x], [c0y, c1y], [c0y, c2y], color='grey', alpha=0.5)
    if isinstance(ordern, int):
        plt.text(textpos[0], textpos[1], "${}$".format(ordern), fontsize=15)
    else:
        plt.text(textpos[0], textpos[1], "${}$".format('%0.2f'%ordern), fontsize=15)



    h_data = 1/N3_c125[:,K_index]
    v_data = N3_c125[:,v_index]
    plt.loglog(h_data, v_data, '-v', color=(1,0,0,0.6), linewidth=0.8)



    h_data = 1/N3_c25[:,K_index]
    v_data = N3_c25[:,v_index]
    plt.loglog(h_data, v_data, '-s', color=(1,0,0,0.3), linewidth=0.8)





    plt.tick_params(which='both', labeltop=False, labelright=False, top=True, right=True)
    plt.tick_params(axis='both', which='minor', direction='in', length=6)
    plt.tick_params(axis='both', which='major', direction='in', length=12)
    plt.tick_params(axis='both', which='both', labelsize=20)
    plt.tick_params(axis='x', which='both', pad=5)
    plt.xlabel('$h$', fontsize=20)
    plt.ylabel(r'$\left\| \varphi^h \right\|_{L^2\text{-}\mathrm{error}}$', fontsize=20)
    # plt.legend(fontsize=13, loc='best', frameon=False)


    plt.yticks([1e-1, 1e-2, 1e-3, 1e-4])
    if do_save:
        plt.savefig(h_what+'_h_convergence_phi_L_2.pdf', bbox_inches='tight')
    plt.show()
    plt.close()



# u E error :  h-convergence
K_index = 0
v_index = 7
plot_hc_u_E_L2_error = 1

if plot_hc_u_E_L2_error:
    plt.figure(figsize=(6,4.5))
    plt.gcf().subplots_adjust(left=0.15)
    plt.gcf().subplots_adjust(bottom=0.15)

    h_data = 1/N1_c0[:,K_index]
    v_data = N1_c0[:,v_index]
    # order = (np.log10(v_data[-1])- np.log10(v_data[-2])) /\
    #                     (np.log10(h_data[-1])-np.log10(h_data[-2]))
    plt.loglog(h_data, v_data, '-o', color=(0,0,0,1), linewidth=0.8)



    h_data = 1/N1_c125[:,K_index]
    v_data = N1_c125[:,v_index]
    plt.loglog(h_data, v_data, '-v', color=(0,0,0,0.6), linewidth=0.8)


    h_data = 1/N1_c25[:,K_index]
    v_data = N1_c25[:,v_index]
    plt.loglog(h_data, v_data, '-s', color=(0,0,0,0.3), linewidth=0.8)



    h_data = 1/N2_c0[:,K_index]
    v_data = N2_c0[:,v_index]
    plt.loglog(h_data, v_data, '-o', color=(0,0,1,1), linewidth=0.8)



    h_data = 1/N2_c125[:,K_index]
    v_data = N2_c125[:,v_index]
    plt.loglog(h_data, v_data, '-v', color=(0,0,1,0.6), linewidth=0.8)



    h_data = 1/N2_c25[:,K_index]
    v_data = N2_c25[:,v_index]
    plt.loglog(h_data, v_data, '-s', color=(0,0,1,0.3), linewidth=0.8)




    h_data = 1/N3_c0[:,K_index]
    v_data = N3_c0[:,v_index]
    plt.loglog(h_data, v_data, '-o', color=(1,0,0,1), linewidth=0.8)




    h_data = 1/N3_c125[:,K_index]
    v_data = N3_c125[:,v_index]
    plt.loglog(h_data, v_data, '-v', color=(1,0,0,0.6), linewidth=0.8)



    h_data = 1/N3_c25[:,K_index]
    v_data = N3_c25[:,v_index]
    plt.loglog(h_data, v_data, '-s', color=(1,0,0,0.3), linewidth=0.8)





    plt.tick_params(which='both', labeltop=False, labelright=False, top=True, right=True)
    plt.tick_params(axis='both', which='minor', direction='in', length=6)
    plt.tick_params(axis='both', which='major', direction='in', length=12)
    plt.tick_params(axis='both', which='both', labelsize=20)
    plt.tick_params(axis='x', which='both', pad=5)
    plt.xlabel('$h$', fontsize=20)
    plt.ylabel(r'$\left\| \nabla\cdot\boldsymbol{u}^h +f^h\right\|_{L^2}$', fontsize=20)
    # plt.legend(fontsize=13, loc='best', frameon=False)

    plt.yticks([1e-13, 1e-14, 1e-15])
    if do_save:
        plt.savefig(h_what+'_h_convergence_E_L_2.pdf', bbox_inches='tight')
    plt.show()
    plt.close()




# phi dH 1 error :  h-convergence
K_index = 0
v_index = 9
plot_hc_phi_dH1_error = 0

if plot_hc_phi_dH1_error and h_what == 'hdMSEM':
    plt.figure(figsize=(6,4.5))
    plt.gcf().subplots_adjust(left=0.15)
    plt.gcf().subplots_adjust(bottom=0.15)

    h_data = 1/N1_c0[:,K_index]
    v_data = N1_c0[:,v_index]
    # order = (np.log10(v_data[-1])- np.log10(v_data[-2])) /\
    #                     (np.log10(h_data[-1])-np.log10(h_data[-2]))
    plt.loglog(h_data, v_data, '-o', color=(0,0,0,1), linewidth=0.8)


    xd_max = np.max(h_data)
    xd_min = np.min(h_data)
    x_range = (xd_max, xd_min)
    c0, c1, c2, textpos, ordern = ___plot_MTIR_plot_order_triangle___(
            'loglog', x_range, {'tp':(0.02, 0.05),'order':1, "p": (0, -0.2)}, h_data, v_data)
    #___plot triangle______________________________________________
    c0x, c0y = c0
    c1x, c1y = c1
    c2x, c2y = c2
    plt.fill_between([c0x, c1x], [c0y, c1y], [c0y, c2y], color='grey', alpha=0.5)
    if isinstance(ordern, int):
        plt.text(textpos[0], textpos[1], "${}$".format(ordern), fontsize=15)
    else:
        plt.text(textpos[0], textpos[1], "${}$".format('%0.2f'%ordern), fontsize=15)


    h_data = 1/N1_c125[:,K_index]
    v_data = N1_c125[:,v_index]
    plt.loglog(h_data, v_data, '-v', color=(0,0,0,0.6), linewidth=0.8)


    h_data = 1/N1_c25[:,K_index]
    v_data = N1_c25[:,v_index]
    plt.loglog(h_data, v_data, '-s', color=(0,0,0,0.3), linewidth=0.8)



    h_data = 1/N2_c0[:,K_index]
    v_data = N2_c0[:,v_index]
    plt.loglog(h_data, v_data, '-o', color=(0,0,1,1), linewidth=0.8)

    x_range = (xd_max, xd_min)
    c0, c1, c2, textpos, ordern = ___plot_MTIR_plot_order_triangle___(
            'loglog', x_range, {'tp':(0.02, 0.2),'order':2, "p": (0, -0.2)}, h_data, v_data)
    #___plot triangle______________________________________________
    c0x, c0y = c0
    c1x, c1y = c1
    c2x, c2y = c2
    plt.fill_between([c0x, c1x], [c0y, c1y], [c0y, c2y], color='grey', alpha=0.5)
    if isinstance(ordern, int):
        plt.text(textpos[0], textpos[1], "${}$".format(ordern), fontsize=15)
    else:
        plt.text(textpos[0], textpos[1], "${}$".format('%0.2f'%ordern), fontsize=15)


    h_data = 1/N2_c125[:,K_index]
    v_data = N2_c125[:,v_index]
    plt.loglog(h_data, v_data, '-v', color=(0,0,1,0.6), linewidth=0.8)



    h_data = 1/N2_c25[:,K_index]
    v_data = N2_c25[:,v_index]
    plt.loglog(h_data, v_data, '-s', color=(0,0,1,0.3), linewidth=0.8)




    h_data = 1/N3_c0[:,K_index]
    v_data = N3_c0[:,v_index]
    plt.loglog(h_data, v_data, '-o', color=(1,0,0,1), linewidth=0.8)

    x_range = (xd_max, xd_min)
    c0, c1, c2, textpos, ordern = ___plot_MTIR_plot_order_triangle___(
            'loglog', x_range, {'tp':(0.02, 0.2),'order':3, "p": (0, -0.3)}, h_data, v_data)
    #___plot triangle______________________________________________
    c0x, c0y = c0
    c1x, c1y = c1
    c2x, c2y = c2
    plt.fill_between([c0x, c1x], [c0y, c1y], [c0y, c2y], color='grey', alpha=0.5)
    if isinstance(ordern, int):
        plt.text(textpos[0], textpos[1], "${}$".format(ordern), fontsize=15)
    else:
        plt.text(textpos[0], textpos[1], "${}$".format('%0.2f'%ordern), fontsize=15)



    h_data = 1/N3_c125[:,K_index]
    v_data = N3_c125[:,v_index]
    plt.loglog(h_data, v_data, '-v', color=(1,0,0,0.6), linewidth=0.8)



    h_data = 1/N3_c25[:,K_index]
    v_data = N3_c25[:,v_index]
    plt.loglog(h_data, v_data, '-s', color=(1,0,0,0.3), linewidth=0.8)





    plt.tick_params(which='both', labeltop=False, labelright=False, top=True, right=True)
    plt.tick_params(axis='both', which='minor', direction='in', length=6)
    plt.tick_params(axis='both', which='major', direction='in', length=12)
    plt.tick_params(axis='both', which='both', labelsize=20)
    plt.tick_params(axis='x', which='both', pad=5)
    plt.xlabel('$h$', fontsize=20)
    plt.ylabel(r'$\left\| \varphi^h \right\|_{\widetilde{H}^1\text{-}\mathrm{error}}$', fontsize=20)
    # plt.legend(fontsize=13, loc='best', frameon=False)


    plt.yticks([1e0, 1e-1, 1e-2, 1e-3])
    if do_save:
        plt.savefig(h_what+'_h_convergence_phi_dH_1.pdf', bbox_inches='tight')
    plt.show()
    plt.close()







# u L_2 error :  p-convergence
N_index = 1
v_index = 3
plot_pc_u_L_2_error = 0

if plot_pc_u_L_2_error:
    plt.figure(figsize=(6,4.5))
    plt.gcf().subplots_adjust(left=0.15)
    plt.gcf().subplots_adjust(bottom=0.15)



    p_data = K1_c0[:,N_index]
    v_data = K1_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(0,0,0,1), linewidth=0.8)

    p_data = K1_c125[:,N_index]
    v_data = K1_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(0,0,0,0.6), linewidth=0.8)

    p_data = K1_c25[:,N_index]
    v_data = K1_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(0,0,0,0.3), linewidth=0.8)



    p_data = K2_c0[:,N_index]
    v_data = K2_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(0,0,1,1), linewidth=0.8)

    p_data = K2_c125[:,N_index]
    v_data = K2_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(0,0,1,0.6), linewidth=0.8)

    p_data = K2_c25[:,N_index]
    v_data = K2_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(0,0,1,0.3), linewidth=0.8)



    p_data = K3_c0[:,N_index]
    v_data = K3_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(1,0,0,1), linewidth=0.8)

    p_data = K3_c125[:,N_index]
    v_data = K3_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(1,0,0,0.6), linewidth=0.8)

    p_data = K3_c25[:,N_index]
    v_data = K3_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(1,0,0,0.3), linewidth=0.8)


    plt.tick_params(which='both', labeltop=False, labelright=False, top=True, right=True)
    plt.tick_params(axis='both', which='minor', direction='in', length=6)
    plt.tick_params(axis='both', which='major', direction='in', length=12)
    plt.tick_params(axis='both', which='both', labelsize=20)
    plt.tick_params(axis='x', which='both', pad=5)
    plt.xlabel('$N$', fontsize=20)
    plt.ylabel(r'$\left\| \boldsymbol{u}^h \right\|_{L^2\text{-}\mathrm{error}}$', fontsize=20)

    # plt.legend(fontsize=18, loc='best', frameon=False)
    if do_save:
        plt.savefig(p_what+'_p_convergence_u_L_2.pdf', bbox_inches='tight')
    plt.show()
    plt.close()


# u H_div error :  p-convergence
N_index = 1
v_index = 4
plot_pc_u_H_div_error = 0

if plot_pc_u_H_div_error:
    plt.figure(figsize=(6,4.5))
    plt.gcf().subplots_adjust(left=0.15)
    plt.gcf().subplots_adjust(bottom=0.15)


    p_data = K1_c0[:,N_index]
    v_data = K1_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(0,0,0,1), linewidth=0.8)

    p_data = K1_c125[:,N_index]
    v_data = K1_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(0,0,0,0.6), linewidth=0.8)

    p_data = K1_c25[:,N_index]
    v_data = K1_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(0,0,0,0.3), linewidth=0.8)



    p_data = K2_c0[:,N_index]
    v_data = K2_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(0,0,1,1), linewidth=0.8)

    p_data = K2_c125[:,N_index]
    v_data = K2_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(0,0,1,0.6), linewidth=0.8)

    p_data = K2_c25[:,N_index]
    v_data = K2_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(0,0,1,0.3), linewidth=0.8)



    p_data = K3_c0[:,N_index]
    v_data = K3_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(1,0,0,1), linewidth=0.8)

    p_data = K3_c125[:,N_index]
    v_data = K3_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(1,0,0,0.6), linewidth=0.8)

    p_data = K3_c25[:,N_index]
    v_data = K3_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(1,0,0,0.3), linewidth=0.8)


    plt.tick_params(which='both', labeltop=False, labelright=False, top=True, right=True)
    plt.tick_params(axis='both', which='minor', direction='in', length=6)
    plt.tick_params(axis='both', which='major', direction='in', length=12)
    plt.tick_params(axis='both', which='both', labelsize=20)
    plt.tick_params(axis='x', which='both', pad=5)
    plt.xlabel('$N$', fontsize=20)
    plt.ylabel(r'$\left\| \boldsymbol{u}^h \right\|_{H(\mathrm{div})\text{-}\mathrm{error}}$', fontsize=20)

    # plt.legend(fontsize=18, loc='best', frameon=False)
    if do_save:
        plt.savefig(p_what+'_p_convergence_u_H_div.pdf', bbox_inches='tight')
    plt.show()
    plt.close()


# phi L_2 error :  p-convergence
N_index = 1
v_index = 5
plot_pc_phi_L_2_error = 0

if plot_pc_phi_L_2_error:
    plt.figure(figsize=(6,4.5))
    plt.gcf().subplots_adjust(left=0.15)
    plt.gcf().subplots_adjust(bottom=0.15)



    p_data = K1_c0[:,N_index]
    v_data = K1_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(0,0,0,1), linewidth=0.8)

    p_data = K1_c125[:,N_index]
    v_data = K1_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(0,0,0,0.6), linewidth=0.8)

    p_data = K1_c25[:,N_index]
    v_data = K1_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(0,0,0,0.3), linewidth=0.8)



    p_data = K2_c0[:,N_index]
    v_data = K2_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(0,0,1,1), linewidth=0.8)

    p_data = K2_c125[:,N_index]
    v_data = K2_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(0,0,1,0.6), linewidth=0.8)

    p_data = K2_c25[:,N_index]
    v_data = K2_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(0,0,1,0.3), linewidth=0.8)



    p_data = K3_c0[:,N_index]
    v_data = K3_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(1,0,0,1), linewidth=0.8)

    p_data = K3_c125[:,N_index]
    v_data = K3_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(1,0,0,0.6), linewidth=0.8)

    p_data = K3_c25[:,N_index]
    v_data = K3_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(1,0,0,0.3), linewidth=0.8)


    plt.tick_params(which='both', labeltop=False, labelright=False, top=True, right=True)
    plt.tick_params(axis='both', which='minor', direction='in', length=6)
    plt.tick_params(axis='both', which='major', direction='in', length=12)
    plt.tick_params(axis='both', which='both', labelsize=20)
    plt.tick_params(axis='x', which='both', pad=5)
    plt.xlabel('$N$', fontsize=20)
    plt.ylabel(r'$\left\| \varphi^h \right\|_{L^2\text{-}\mathrm{error}}$', fontsize=20)

    # plt.legend(fontsize=18, loc='best', frameon=False)
    if do_save:
        plt.savefig(p_what+'_p_convergence_phi_L_2.pdf', bbox_inches='tight')
    plt.show()
    plt.close()


# E L_2 error :  p-convergence
N_index = 1
v_index = 7
plot_pc_E_L_2_error = 1

if plot_pc_E_L_2_error:
    plt.figure(figsize=(6,4.5))
    plt.gcf().subplots_adjust(left=0.15)
    plt.gcf().subplots_adjust(bottom=0.15)



    p_data = K1_c0[:,N_index]
    v_data = K1_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(0,0,0,1), linewidth=0.8)

    p_data = K1_c125[:,N_index]
    v_data = K1_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(0,0,0,0.6), linewidth=0.8)

    p_data = K1_c25[:,N_index]
    v_data = K1_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(0,0,0,0.3), linewidth=0.8)



    p_data = K2_c0[:,N_index]
    v_data = K2_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(0,0,1,1), linewidth=0.8)

    p_data = K2_c125[:,N_index]
    v_data = K2_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(0,0,1,0.6), linewidth=0.8)

    p_data = K2_c25[:,N_index]
    v_data = K2_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(0,0,1,0.3), linewidth=0.8)



    p_data = K3_c0[:,N_index]
    v_data = K3_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(1,0,0,1), linewidth=0.8)

    p_data = K3_c125[:,N_index]
    v_data = K3_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(1,0,0,0.6), linewidth=0.8)

    p_data = K3_c25[:,N_index]
    v_data = K3_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(1,0,0,0.3), linewidth=0.8)


    plt.tick_params(which='both', labeltop=False, labelright=False, top=True, right=True)
    plt.tick_params(axis='both', which='minor', direction='in', length=6)
    plt.tick_params(axis='both', which='major', direction='in', length=12)
    plt.tick_params(axis='both', which='both', labelsize=20)
    plt.tick_params(axis='x', which='both', pad=5)
    plt.xlabel('$N$', fontsize=20)
    plt.ylabel(r'$\left\| \nabla\cdot\boldsymbol{u}^h +f^h\right\|_{L^2}$', fontsize=20)

    # plt.legend(fontsize=18, loc='best', frameon=False)
    if do_save:
        plt.savefig(p_what+'_p_convergence_E_L_2.pdf', bbox_inches='tight')
    plt.show()
    plt.close()



# phi dH_1 error :  p-convergence
N_index = 1
v_index = 9
plot_pc_phi_dH_1_error = 1

if plot_pc_phi_dH_1_error and p_what == 'hdMSEM':
    plt.figure(figsize=(6,4.5))
    plt.gcf().subplots_adjust(left=0.15)
    plt.gcf().subplots_adjust(bottom=0.15)



    p_data = K1_c0[:,N_index]
    v_data = K1_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(0,0,0,1), linewidth=0.8)

    p_data = K1_c125[:,N_index]
    v_data = K1_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(0,0,0,0.6), linewidth=0.8)

    p_data = K1_c25[:,N_index]
    v_data = K1_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(0,0,0,0.3), linewidth=0.8)



    p_data = K2_c0[:,N_index]
    v_data = K2_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(0,0,1,1), linewidth=0.8)

    p_data = K2_c125[:,N_index]
    v_data = K2_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(0,0,1,0.6), linewidth=0.8)

    p_data = K2_c25[:,N_index]
    v_data = K2_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(0,0,1,0.3), linewidth=0.8)



    p_data = K3_c0[:,N_index]
    v_data = K3_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(1,0,0,1), linewidth=0.8)

    p_data = K3_c125[:,N_index]
    v_data = K3_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(1,0,0,0.6), linewidth=0.8)

    p_data = K3_c25[:,N_index]
    v_data = K3_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(1,0,0,0.3), linewidth=0.8)


    plt.tick_params(which='both', labeltop=False, labelright=False, top=True, right=True)
    plt.tick_params(axis='both', which='minor', direction='in', length=6)
    plt.tick_params(axis='both', which='major', direction='in', length=12)
    plt.tick_params(axis='both', which='both', labelsize=20)
    plt.tick_params(axis='x', which='both', pad=5)
    plt.xlabel('$N$', fontsize=20)
    plt.ylabel(r'$\left\| \varphi^h\right\|_{\widetilde{H}^1\text{-}\mathrm{error}}$', fontsize=20)

    # plt.legend(fontsize=18, loc='best', frameon=False)
    if do_save:
        plt.savefig(p_what+'_p_convergence_phi_dH_1.pdf', bbox_inches='tight')
    plt.show()
    plt.close()





# f L_2 error :  p-convergence
N_index = 1
v_index = 6
plot_pc_f_L_2_error = 0

if plot_pc_f_L_2_error and p_what == 'MSEM':
    plt.figure(figsize=(6,4.5))
    plt.gcf().subplots_adjust(left=0.15)
    plt.gcf().subplots_adjust(bottom=0.15)



    p_data = K1_c0[:,N_index]
    v_data = K1_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(0,0,0,1), linewidth=0.8)

    p_data = K1_c125[:,N_index]
    v_data = K1_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(0,0,0,0.6), linewidth=0.8)

    p_data = K1_c25[:,N_index]
    v_data = K1_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(0,0,0,0.3), linewidth=0.8)



    p_data = K2_c0[:,N_index]
    v_data = K2_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(0,0,1,1), linewidth=0.8)

    p_data = K2_c125[:,N_index]
    v_data = K2_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(0,0,1,0.6), linewidth=0.8)

    p_data = K2_c25[:,N_index]
    v_data = K2_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(0,0,1,0.3), linewidth=0.8)



    p_data = K3_c0[:,N_index]
    v_data = K3_c0[:,v_index]
    plt.semilogy(p_data, v_data, '-o', color=(1,0,0,1), linewidth=0.8)

    p_data = K3_c125[:,N_index]
    v_data = K3_c125[:,v_index]
    plt.semilogy(p_data, v_data, '-v', color=(1,0,0,0.6), linewidth=0.8)

    p_data = K3_c25[:,N_index]
    v_data = K3_c25[:,v_index]
    plt.semilogy(p_data, v_data, '-s', color=(1,0,0,0.3), linewidth=0.8)


    plt.tick_params(which='both', labeltop=False, labelright=False, top=True, right=True)
    plt.tick_params(axis='both', which='minor', direction='in', length=6)
    plt.tick_params(axis='both', which='major', direction='in', length=12)
    plt.tick_params(axis='both', which='both', labelsize=20)
    plt.tick_params(axis='x', which='both', pad=5)
    plt.xlabel('$N$', fontsize=20)
    plt.ylabel(r'$\left\| f^h \right\|_{L^2\text{-}\mathrm{error}}$', fontsize=20)

    # plt.legend(fontsize=18, loc='best', frameon=False)
    if do_save:
        plt.savefig(p_what+'_p_convergence_f_L_2.pdf', bbox_inches='tight')
    plt.show()
    plt.close()
